unit OpticalArea;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, ExtCtrls, Controls, Dialogs, Forms, Menus;

type
  TOpticalArea = class(TShape)
  private
    inReposition:   Boolean;
    PopMenuCreated: Boolean;
    oldPos :        TPoint;
  public
    Count: Integer;
    RefractiveIndex: Real;
    constructor Create(TheOwner: TComponent); override;
    destructor  Destroy; override;
    procedure   ControlMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure   ControlMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure   ControlMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure   ControlMouseEnter(Sender: TObject);
    procedure   ControlDblClick(Sender: TObject);
    procedure   CreatePopupMenu();
    procedure   CreateColorDialog(Sender: TObject);
    procedure   CreateSettingsDialog(Sender: TObject);
    function    IsFloat(var s: string): boolean;
    function    IsInteger(var s: string): boolean;
  end;

implementation

var
  OptAreaCnt: Integer = 0;    // class variable for counting/indexing all instances of TOpticalAreas

constructor TOpticalArea.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  inc(OptAreaCnt);
  Name            := 'OpticalArea' + IntToStr(OptAreaCnt);
  Count           := 0;
  RefractiveIndex := 1.5;
  PopMenuCreated  := false;
  OnMouseDown     := @ControlMouseDown;
  OnMouseMove     := @ControlMouseMove;
  OnMouseUp       := @ControlMouseUp;
  OnMouseEnter    := @ControlMouseEnter;
  OnDblClick      := @ControlDblClick;
end;

destructor  TOpticalArea.Destroy;
begin
  inherited Destroy;
end;

procedure TOpticalArea.ControlMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Sender is TOpticalArea) then
  begin
    if Button = mbLeft then
    begin
      inReposition := True;
      oldPos       := Point(X, Y);
    end
    else if Button = mbRight then
      CreatePopupMenu();
  end;
end; (*ControlMouseDown*)

procedure TOpticalArea.ControlMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
const
  minWidth = 5;
  minHeight = 5;
var
  newPos: TPoint;
  frmPoint : TPoint;
begin
  if inReposition then
  begin
    with TControl(Sender) do
    begin
      newPos := Point(X, Y);
      if ssShift in Shift then
      begin //resize
        Screen.Cursor := crSizeNWSE;
        frmPoint := Point(Width + X - oldPos.X, Height + Y - oldPos.Y);
        if frmPoint.X > minWidth then
          Width := frmPoint.X;
        if frmPoint.Y > minHeight then
          Height := frmPoint.Y;
        oldPos := newPos;
      end
      else //move
      begin
        Screen.Cursor := crSize;
        Left := Left - oldPos.X + newPos.X;
        Top := Top - oldPos.Y + newPos.Y;
      end;
    end;
  end;
end; (*ControlMouseMove*)

procedure TOpticalArea.ControlMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if inReposition then
  begin
    Screen.Cursor := crDefault;
    //ReleaseCapture;
    inReposition := False;
  end;
end; (*ControlMouseUp*)

procedure TOpticalArea.ControlMouseEnter(Sender: TObject);
var
  s: String;

begin
  s := Name + ', Height: ' + IntToStr(Height) + ', Width: ' + IntToStr(Width) + ', n: ' + FloatToStr(RefractiveIndex);
  Canvas.TextOut(1,1, s);
end; (*ControlMouseEnter*)

procedure TOpticalArea.ControlDblClick(Sender: TObject);
begin
  if TOpticalArea(Sender).ClassType = TOpticalArea then
  begin
    inReposition := False;
    TOpticalArea(Sender).Free;
    //Repaint;
  end;
end; (*ControlDblClick*)

procedure TOpticalArea.CreatePopupMenu();
var
  PopMenu:     TPopupMenu;
  NewItem:     TMenuItem;

begin
  if PopMenuCreated = false then
  begin
    PopMenu   := TPopupMenu.Create(Self);
    PopupMenu := PopMenu; // PopMenu dem Shape(aus TControl) zu weisen

    NewItem         := TMenuItem.Create(Self);
    NewItem.Caption := 'Change Color';
    NewItem.Name    := 'ChangeColor';
    NewItem.OnClick := @CreateColorDialog; // Eventhandler zuweisen
    PopMenu.Items.Add(NewItem);

    NewItem         := TMenuItem.Create(Self);
    NewItem.Caption := 'Change Refractive Index';
    NewItem.Name    := 'RefractiveIndex';
    NewItem.OnClick := @CreateSettingsDialog; // Eventhandler zuweisen
    PopMenu.Items.Add(NewItem);

    NewItem         := TMenuItem.Create(Self);
    NewItem.Caption := 'Change Height';
    NewItem.Name    := 'Height';
    NewItem.OnClick := @CreateSettingsDialog; // Eventhandler zuweisen
    PopMenu.Items.Add(NewItem);

    NewItem         := TMenuItem.Create(Self);
    NewItem.Caption := 'Change Width';
    NewItem.Name    := 'Width';
    NewItem.OnClick := @CreateSettingsDialog; // Eventhandler zuweisen
    PopMenu.Items.Add(NewItem);

    PopMenuCreated  := true;
  end;
end;

procedure TOpticalArea.CreateColorDialog(Sender: TObject);
var
  ColorDialog: TColorDialog;

begin
  ColorDialog := TColorDialog.Create(Self);
  if ColorDialog.Execute then
    Brush.Color := (ColorDialog.Color);
end;

procedure TOpticalArea.CreateSettingsDialog(Sender: TObject);
var
  s: string;

begin
  if TMenuItem(Sender).Name = 'RefractiveIndex' then
  begin
    s := FloatToStr(RefractiveIndex);
    while (not(InputQuery('Optical Area Settings', 'Refractive Index:', s)) or not(IsFloat(s))) do;
    RefractiveIndex := StrToFloat(s);
  end;
  if TMenuItem(Sender).Name = 'Height' then
  begin
    s := IntToStr(Height);
    while (not(InputQuery('Optical Area Settings', 'Height:', s)) or not(IsInteger(s))) do;
    Height := StrToInt(s);
  end;
  if TMenuItem(Sender).Name = 'Width' then
  begin
    s := IntToStr(Width);
    while (not(InputQuery('Optical Area Settings', 'Width:', s)) or not(IsInteger(s))) do;
    Width := StrToInt(s);
  end;

end;

function TOpticalArea.IsFloat(var s: string): boolean;
var
  float: real;

begin
// Prüfung ob Float
  if TryStrToFloat(s, float) then
    IsFloat := true
  else
    IsFloat := false;
end;

function TOpticalArea.IsInteger(var s: string): boolean;
var
  int: integer;

begin
// Prüfung ob Integer
  if TryStrToInt(s, int) then
    IsInteger := true
  else
    IsInteger := false;
end;

end.


